"use strict";

module.exports = function () {
  return {
    name: "transform-merge-sibling-variables",
    visitor: {
      VariableDeclaration: {
        enter: [
        // concat variables of the same kind with their siblings
        function (path) {
          if (!path.inList) {
            return;
          }

          var node = path.node;


          while (true) {
            var sibling = path.getSibling(path.key + 1);
            if (!sibling.isVariableDeclaration({ kind: node.kind })) {
              break;
            }

            node.declarations = node.declarations.concat(sibling.node.declarations);
            sibling.remove();
          }
        },

        // concat `var` declarations next to for loops with it's initialisers.
        // block-scoped `let` and `const` are not moved because the for loop
        // is a different block scope.
        function (path) {
          if (!path.inList) {
            return;
          }

          var node = path.node;

          if (node.kind !== "var") {
            return;
          }

          var next = path.getSibling(path.key + 1);
          if (!next.isForStatement()) {
            return;
          }

          var init = next.get("init");
          if (!init.isVariableDeclaration({ kind: node.kind })) {
            return;
          }

          init.node.declarations = node.declarations.concat(init.node.declarations);
          path.remove();
        }]
      }
    }
  };
};